@@ -4,7 +4,7 @@
  *
  * copyright (c) 2004-6, Karl W Broman, Johns Hopkins University
  *
- * last modified Feb, 2006
+ * last modified Oct, 2006
  * first written Dec, 2004
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -96,7 +96,7 @@ void R_scantwo_1chr_binary_em(int *n_ind, int *n_pos, int *n_gen,
  * Result       Result matrix of size [n_pos x n_pos]; the lower
  *              triangle (row > col) contains the joint LODs while 
  *              the upper triangle (row < col) contains the LODs for 
- *              testing epistasis.
+ *              an additive model.
  *              Note: indexed as Result[col][row]
  *
  * maxit        Maximum number of iterations for EM
@@ -261,8 +261,8 @@ void scantwo_1chr_binary_em(int n_ind, int n_pos, int n_gen,
 	Rprintf("\n");
       }
 
-      Result[i2][i1] = llik[1]-llik[0];
-      Result[i1][i2] = -llik[1];  
+      Result[i2][i1] = -llik[0];
+      Result[i1][i2] = -llik[1];
 
     } /* position 2 */
   } /* position 1 */
@@ -282,16 +282,16 @@ void R_scantwo_2chr_binary_em(int *n_ind, int *n_pos1, int *n_pos2,
 			      double *genoprob2, double *addcov, int *n_addcov, 
 			      double *intcov, int *n_intcov, 
 			      int *pheno, double *start,
-			      double *result_full, double *result_int,
+			      double *result_full, double *result_add,
 			      int *maxit, double *tol, int *verbose)
 {
-  double **Result_full, **Result_int, **Addcov, **Intcov;
+  double **Result_full, **Result_add, **Addcov, **Intcov;
   double ***Genoprob1, ***Genoprob2;
 
   reorg_genoprob(*n_ind, *n_pos1, *n_gen1, genoprob1, &Genoprob1);
   reorg_genoprob(*n_ind, *n_pos2, *n_gen2, genoprob2, &Genoprob2);
   reorg_errlod(*n_pos1, *n_pos2, result_full, &Result_full);
-  reorg_errlod(*n_pos1, *n_pos2, result_int, &Result_int);
+  reorg_errlod(*n_pos1, *n_pos2, result_add, &Result_add);
 
   /* reorganize addcov and intcov (if they are not empty) */
   if(*n_addcov > 0) reorg_errlod(*n_ind, *n_addcov, addcov, &Addcov);
@@ -300,7 +300,7 @@ void R_scantwo_2chr_binary_em(int *n_ind, int *n_pos1, int *n_pos2,
   scantwo_2chr_binary_em(*n_ind, *n_pos1, *n_pos2, *n_gen1, *n_gen2,
 			 Genoprob1, Genoprob2, Addcov, *n_addcov, 
 			 Intcov, *n_intcov, pheno, start,
-			 Result_full, Result_int, 
+			 Result_full, Result_add, 
 			 *maxit, *tol, *verbose);
 }
 
@@ -342,8 +342,8 @@ void R_scantwo_2chr_binary_em(int *n_ind, int *n_pos1, int *n_pos2,
  *              containing the joint LODs
  *              Note: indexed as Result[pos2][pos1]
  *
- * Result_int   Result matrix of size [n_pos2 x n_pos1] 
- *              containing the LODs testing interactions
+ * Result_add   Result matrix of size [n_pos2 x n_pos1] 
+ *              containing the LODs for additive models
  *              also indexed as Result[pos2][pos1]
  *
  * maxit        Maximum number of iterations for EM
@@ -362,7 +362,7 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 			    int n_gen2, double ***Genoprob1, double ***Genoprob2,
 			    double **Addcov, int n_addcov, double **Intcov, 
 			    int n_intcov, int *pheno, double *start,
-			    double **Result_full, double **Result_int, 
+			    double **Result_full, double **Result_add, 
 			    int maxit, double tol, int verbose)
 {
   int error_flag, i1, i2, k1, k2, j, m, n_col[2], nit[2], r, flag=0;
@@ -488,7 +488,7 @@ void scantwo_2chr_binary_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 	Rprintf("\n");
       }
 
-      Result_int[i2][i1] = llik[1]-llik[0];
+      Result_add[i2][i1] = -llik[0];
       Result_full[i2][i1] = -llik[1];  
     } /* position 2 */
   } /* position 1 */
